@@ -1,6 +1,7 @@
+import math
 from dataclasses import dataclass, field
 from enum import auto
-from typing import Dict, List, Optional, Union
+from typing import Dict, List, Optional, Tuple, Union
 
 from strenum import LowercaseStrEnum, StrEnum
 
@@ -21,7 +22,7 @@ class SequenceResidue: ...  # noqa: E701
 class Residue: ...  # noqa: E701
 
 
-# something like this migh be nice
+# something like this might be nice
 # but we would need updates...
 # class ResidueAssignmentState(StrEnum):
 #     UNASSIGNED = auto()
@@ -299,6 +300,33 @@ class DiffusionModel(StrEnum):
     ELLIPSOID = auto()
 
 
+# when we get beyond 3.9 support this should change to
+# Vector3D: TypeAlias = Tuple[float, float, float]
+# and when we get beyon 3.11 support this should change to
+# type Vector3D = Tuple[float, float, float]
+Vector3D = Tuple[float, float, float]
+
+
+@dataclass(frozen=True)
+class RdcTensorFrameData:
+    da: float
+    dr: float
+
+    eigen_vector_x: Vector3D
+    eigen_vector_y: Vector3D
+    eigen_vector_z: Vector3D
+
+    def is_defined(self):
+        da_ok = not math.isnan(self.da)
+        dr_ok = not math.isnan(self.dr)
+        eigen_x_ok = not any(math.isnan(elem) for elem in self.eigen_vector_x)
+        eigen_y_ok = not any(math.isnan(elem) for elem in self.eigen_vector_y)
+        eigen_z_ok = not any(math.isnan(elem) for elem in self.eigen_vector_z)
+        eigen_ok = eigen_x_ok and eigen_y_ok and eigen_z_ok
+
+        return da_ok and dr_ok and eigen_ok
+
+
 @dataclass(frozen=True)
 class TensorFrame:
     d_iso: float = None
