@@ -6,13 +6,13 @@ from pynmrstar import Entry, Saveframe
 
 from nef_pipelines.lib.sequence_lib import (
     BadResidue,
+    chains_from_frames,
     count_residues,
-    frame_to_chains,
     get_chain_code_iter,
     get_chain_starts,
     offset_chain_residues,
     sequence_3let_to_sequence_residues,
-    sequence_from_frame,
+    sequences_from_frames,
     translate_1_to_3,
 )
 from nef_pipelines.lib.structures import SequenceResidue
@@ -131,7 +131,7 @@ TEST_DATA_MULTI_CHAIN = """
 def test_list_chains():
 
     test_frame = Saveframe.from_string(TEST_DATA_MULTI_CHAIN)
-    chains = frame_to_chains(test_frame)
+    chains = chains_from_frames(test_frame)
 
     assert chains == list(["A", "B", "C"])
 
@@ -163,7 +163,7 @@ def test_list_chains_no_chains():
 
     """
     test_frame = Saveframe.from_string(TEST_DATA)
-    chains = frame_to_chains(test_frame)
+    chains = chains_from_frames(test_frame)
 
     assert chains == list([])
 
@@ -254,14 +254,10 @@ def test_sequence_from_frame():
 
     frames = Entry.from_file(path).get_saveframes_by_category("nef_molecular_system")
 
-    sequence = sequence_from_frame(frames[0], chain_codes_to_select=["B", "C"])
+    sequence = sequences_from_frames(frames[0], chain_codes_to_select=["B", "C"])
 
     EXPECTED = [
-        SequenceResidue(
-            chain_code="B",
-            sequence_code=5,
-            residue_name="ARG",
-        ),
+        SequenceResidue(chain_code="B", sequence_code=5, residue_name="ARG"),
         SequenceResidue(chain_code="B", sequence_code=6, residue_name="GLN"),
         SequenceResidue(chain_code="C", sequence_code=7, residue_name="PRO"),
     ]
@@ -269,12 +265,24 @@ def test_sequence_from_frame():
     assert sequence == EXPECTED
 
 
+def test_sequence_from_frame_empty():
+    path = path_in_test_data(__file__, "multi_chain.nef")
+
+    frames = Entry.from_file(path).get_saveframes_by_category("nef_molecular_system")
+
+    sequence = sequences_from_frames(frames[0], chain_codes_to_select=["D", "E"])
+
+    EXPECTED = []
+
+    assert sequence == EXPECTED
+
+
 def test_sequence_from_frame_all():
     path = path_in_test_data(__file__, "multi_chain.nef")
 
     frames = Entry.from_file(path).get_saveframes_by_category("nef_molecular_system")
 
-    sequence = sequence_from_frame(frames[0])
+    sequence = sequences_from_frames(frames[0])
 
     EXPECTED = [
         SequenceResidue(chain_code="A", sequence_code=3, residue_name="HIS"),
