@@ -18,16 +18,17 @@ def test_2_distances():
     sequence_path = path_in_test_data(__file__, "3a_ab.neff")
     dihedrals_path = path_in_test_data(__file__, "test_2_distances.tbl")
 
-    nef_sequence = open(sequence_path, "r").read()
+    with open(sequence_path, "r") as fh:
+        nef_sequence = fh.read()
 
     args = [dihedrals_path]
     result = run_and_report(app, args, input=nef_sequence)
 
     EXPECTED = """\
         save_nef_distance_restraint_list_test_2_distances                               # noqa: E501
-            _nef_distance_restraint.sf_category     nef_distance_restraint_list
-            _nef_distance_restraint.sf_framecode    nef_distance_restraint_list_test_2_distances
-            _nef_distance_restraint.potential_type  undefined
+            _nef_distance_restraint_list.sf_category     nef_distance_restraint_list
+            _nef_distance_restraint_list.sf_framecode    nef_distance_restraint_list_test_2_distances
+            _nef_distance_restraint_list.potential_type  undefined
 
             loop_
                 _nef_distance_restraint.index
@@ -45,8 +46,8 @@ def test_2_distances():
                 _nef_distance_restraint.target_value
                 _nef_distance_restraint.lower_limit
                 _nef_distance_restraint.upper_limit
-                1   1   .   AAAA   1   .   HN   AAAA   2   .   HDA#   1.0   2.6   2.0   8.2
-                2   2   .   AAAA   2   .   HN   AAAA   3   .   HD1#   1.0   4.5   3.0   10.7
+                1   1   .   AAAA   1   .   HN   AAAA   2   .   HDA#   1.0   2.6   0.6   5.6
+                2   2   .   AAAA   2   .   HN   AAAA   3   .   HD1#   1.0   4.5   1.5   6.2
 
            stop_
         save_
@@ -65,7 +66,8 @@ def test_2_distances_bad():
     sequence_path = path_in_test_data(__file__, "3a_ab.neff")
     distances_path = path_in_test_data(__file__, "test_2_distances_bad.tbl")
 
-    nef_sequence = open(sequence_path, "r").read()
+    with open(sequence_path, "r") as fh:
+        nef_sequence = fh.read()
 
     args = [distances_path]
     result = run_and_report(app, args, input=nef_sequence, expected_exit_code=1)
@@ -78,16 +80,17 @@ def test_2_dihstances_no_segids():
     sequence_path = path_in_test_data(__file__, "3a_ab.neff")
     distances_path = path_in_test_data(__file__, "test_2_distances_no_segids.tbl")
 
-    nef_sequence = open(sequence_path, "r").read()
+    with open(sequence_path, "r") as fh:
+        nef_sequence = fh.read()
 
     args = [distances_path, "--chains", "AAAA"]
     result = run_and_report(app, args, input=nef_sequence)
 
     EXPECTED = """\
         save_nef_distance_restraint_list_test_2_distances_no_segids                               # noqa: E501
-            _nef_distance_restraint.sf_category     nef_distance_restraint_list
-            _nef_distance_restraint.sf_framecode    nef_distance_restraint_list_test_2_distances_no_segids
-            _nef_distance_restraint.potential_type  undefined
+            _nef_distance_restraint_list.sf_category     nef_distance_restraint_list
+            _nef_distance_restraint_list.sf_framecode    nef_distance_restraint_list_test_2_distances_no_segids
+            _nef_distance_restraint_list.potential_type  undefined
 
             loop_
                 _nef_distance_restraint.index
@@ -105,8 +108,8 @@ def test_2_dihstances_no_segids():
                 _nef_distance_restraint.target_value
                 _nef_distance_restraint.lower_limit
                 _nef_distance_restraint.upper_limit
-                1   1   .   AAAA   1   .   HN   AAAA   2   .   HDA#   1.0   2.6   2.0   8.2
-                2   2   .   AAAA   2   .   HN   AAAA   3   .   HD1#   1.0   4.5   3.0   10.7
+                1   1   .   AAAA   1   .   HN   AAAA   2   .   HDA#   1.0   2.6   0.6   5.6
+                2   2   .   AAAA   2   .   HN   AAAA   3   .   HD1#   1.0   4.5   1.5   6.2
 
            stop_
         save_
@@ -125,16 +128,17 @@ def test_2_distances_overriding_segids():
     sequence_path = path_in_test_data(__file__, "3a_ab.neff")
     distances_path = path_in_test_data(__file__, "test_2_distances.tbl")
 
-    nef_sequence = open(sequence_path, "r").read()
+    with open(sequence_path, "r") as fh:
+        nef_sequence = fh.read()
 
     args = [distances_path, "--chains", "BBBB", "--use-chains"]
     result = run_and_report(app, args, input=nef_sequence)
 
     EXPECTED = """\
         save_nef_distance_restraint_list_test_2_distances                               # noqa: E501
-            _nef_distance_restraint.sf_category     nef_distance_restraint_list
-            _nef_distance_restraint.sf_framecode    nef_distance_restraint_list_test_2_distances
-            _nef_distance_restraint.potential_type  undefined
+            _nef_distance_restraint_list.sf_category     nef_distance_restraint_list
+            _nef_distance_restraint_list.sf_framecode    nef_distance_restraint_list_test_2_distances
+            _nef_distance_restraint_list.potential_type  undefined
 
             loop_
                 _nef_distance_restraint.index
@@ -152,8 +156,8 @@ def test_2_distances_overriding_segids():
                 _nef_distance_restraint.target_value
                 _nef_distance_restraint.lower_limit
                 _nef_distance_restraint.upper_limit
-                1   1   .   BBBB   1   .   HN   BBBB   2   .   HDA#   1.0   2.6   2.0   8.2
-                2   2   .   BBBB   2   .   HN   BBBB   3   .   HD1#   1.0   4.5   3.0   10.7
+                1   1   .   BBBB   1   .   HN   BBBB   2   .   HDA#   1.0   2.6   0.6   5.6
+                2   2   .   BBBB   2   .   HN   BBBB   3   .   HD1#   1.0   4.5   1.5   6.2
 
            stop_
         save_
