@@ -1,10 +1,10 @@
-from enum import auto
 from pathlib import Path
 from typing import List
 
 import typer
+
+# from lazy_import import lazy_module
 from pynmrstar import Entry, Saveframe
-from strenum import StrEnum
 
 from nef_pipelines.lib.nef_lib import (
     NEF_PIPELINES_PREFIX,
@@ -21,21 +21,23 @@ from nef_pipelines.tools.fit.fit_lib import (
     _series_frame_to_id_series_data,
 )
 
-streamfitter_install_failure = None
 try:
-    from streamfitter.fitter import ErrorPropogation, fitter
 
-    stream_fitter = fitter
+    from streamfitter.error_propogation import ErrorPropogation
 
 except ImportError as e:
-    streamfitter_intall_failure = e
+    from enum import auto
+
+    from strenum import StrEnum
 
-    # this is partial copy of the enum to avoid errors
     class ErrorPropogation(StrEnum):
-        PROPOGATION = "error stream fitter package not installed"
-        ERROR_STREAM_FITTER_NOT_INSTALLED = auto()
+        PROPOGATION = auto()
+        JACKNIFE = auto()
+        BOOTSTRAP = auto()
+
+    streamfitter = None
+    stream_fitter_import_error = str(e)
 
-    stream_fitter = None
 
 NAMESPACE = NEF_PIPELINES_PREFIX
 
@@ -53,7 +55,7 @@ def exponential(
         ErrorPropogation.PROPOGATION,
         "-e",
         "--error-method",
-        help="error propogation method",
+        help="error propagation method",
     ),
     cycles: int = typer.Option(
         1000,
@@ -111,10 +113,16 @@ def pipe(
     seed: int,
 ) -> Entry:
 
-    if stream_fitter is None:
+    try:
+        if streamfitter:
+            fitter = streamfitter.fitter
+        else:
+            raise ImportError(stream_fitter_import_error)
+    except ImportError as e:
+
         msg = f"""
                 error the package streamfitter is not installed or is not importing properly
-                the error was {streamfitter_intall_failure}
+                the error was {e}
             """
 
         exit_error(msg)
@@ -129,7 +137,7 @@ def pipe(
             for id, series_datum in id_series_data.items()
         }
 
-        results = stream_fitter(id_xy_data, error_method, cycles, noise_level, seed)
+        results = fitter(id_xy_data, error_method, cycles, noise_level, seed)
 
         fits = results["fits"]
         monte_carlo_errors = results["monte_carlo_errors"]
