@@ -14,6 +14,7 @@ from nef_pipelines.lib.nef_frames_lib import (
     SHIFT_LIST_FRAME_CATEGORY,
 )
 from nef_pipelines.lib.nef_lib import (
+    UNUSED,
     read_entry_from_file_or_stdin_or_exit_error,
     select_frames_by_name,
 )
@@ -45,7 +46,7 @@ SPECTRUM_NAME_TEMPLATE = "synthetic_{spectrum}"
 SHIFT_FRAMES_HELP = """\
     the names of the shift frames to use, this can be a comma separated list of name or the option can be called
     called multiple times. Wild cards are allowed. If no match is found wild cards are checked as well unless
-    --exact is set
+    --exact is set. If no shift frames are selected the default chemical shift list frame is used
 """
 
 SPECTRA_HELP = f"""
@@ -57,6 +58,7 @@ NAME_TEMPLATE_HELP = """
 """
 
 
+# TODO: add a flag to list spectra and groups...
 @simulate_app.command()
 def peaks(
     shift_frame_selectors: List[str] = typer.Option(
@@ -77,7 +79,7 @@ def peaks(
     spectra: List[str] = typer.Argument(None, help=SPECTRA_HELP),
 ):
     """-  make a set of peaks for an hsqc, 13C direct detect or triple resonance spectrum from a list of shifts
-    [alpha for non cpn peak lists and 13C detect]"""
+    [alpha for non ccpn peak lists and 13C detect]"""
 
     spectra = [spectrum.replace("-", "_") for spectrum in spectra]
     if not spectra:
@@ -87,7 +89,7 @@ def peaks(
 
     shift_frame_selectors = (
         [
-            "*",
+            "nef_chemical_shift_list_default",
         ]
         if not shift_frame_selectors
         else parse_comma_separated_options(shift_frame_selectors)
@@ -112,91 +114,6 @@ def peaks(
     print(entry)
 
 
-def _strs_to_experiment_types_or_exit_error(spectra):
-    result = []
-    spectrum_names = set([experiment.value for experiment in ExperimentType])
-    for spectum in spectra:
-
-        found = False
-        if spectum in spectrum_names:
-            result.append(ExperimentType[spectum])
-            found = True
-
-        experiment_types_upper = [
-            experiment_type.name.upper() for experiment_type in ExperimentType
-        ]
-        upper_experiments_to_experiments = {
-            experiment_type.name.upper(): experiment_type
-            for experiment_type in ExperimentType
-        }
-        counts = Counter(experiment_types_upper)
-
-        if not found:
-            spectrum_upper = spectum.upper()
-            if spectrum_upper in experiment_types_upper and counts[spectrum_upper] == 1:
-                result.append(upper_experiments_to_experiments[spectrum_upper])
-                found = True
-            if spectrum_upper in experiment_types_upper and counts[spectrum_upper] > 1:
-                degenerate_spectra = [
-                    experiment_type.name
-                    for experiment_type in ExperimentType
-                    if experiment_type.upper() == spectrum_upper
-                ]
-                _exit_degnerate_spectrum_type(
-                    spectum, list(ExperimentType), degenerate_spectra
-                )
-
-        if not found:
-            _exit_bad_spectrum_type(spectum, list(ExperimentType))
-
-    return result
-
-
-def _update_spectra_with_groups(spectra):
-
-    spectra = set(spectra)
-    to_update = []
-    for experiment_type in spectra:
-        if experiment_type in EXPERIMENT_GROUPS:
-            to_update.append(experiment_type)
-
-    for experiment_type in to_update:
-        spectra.update(EXPERIMENT_GROUPS[experiment_type])
-        spectra.remove(experiment_type)
-
-    return spectra
-
-
-def _exit_bad_spectrum_type(spectrum, known_spectra):
-    msg = """
-        the spectrum type {spectrum} is not know, available spectrum types are
-
-        {spectrum_types}
-    """
-    msg = dedent(msg)
-
-    spectrum_types = strings_to_tabulated_terminal_sensitive(known_spectra)
-    spectrum_types = indent(spectrum_types, FOUR_SPACES)
-
-    exit_error(f(msg))
-
-
-def _exit_degnerate_spectrum_type(spectrum, known_spectra, degenerate_spectra):
-    degenerate_spectra = ", ".join(degenerate_spectra)
-    msg = """
-            the spectrum type {spectrum} doesn't exactly match a spectrum type in the list below
-            and has more than possible match when case is ignored [{degenerate_spectra}]
-
-            {spectrum_types}
-        """
-    msg = dedent(msg)
-
-    spectrum_types = strings_to_tabulated_terminal_sensitive(known_spectra)
-    spectrum_types = indent(spectrum_types, FOUR_SPACES)
-
-    exit_error(f(msg))
-
-
 def pipe(
     entry: Entry,
     shift_frame_selectors: List[str],
@@ -259,6 +176,30 @@ def _sign_of_value(value):
     return result
 
 
+def _average_atom_set_shifts(atom_set_shifts):
+    shifts_by_atom = {}
+    for atom_set_shift in atom_set_shifts:
+        residue = atom_set_shift.atom.residue
+        atom_key = (
+            residue.chain_code,
+            residue.sequence_code,
+            residue.offset,
+            atom_set_shift.atom.atom_name,
+        )
+        shifts_by_atom.setdefault(atom_key, []).append(atom_set_shift)
+
+    result = []
+    for key, values in shifts_by_atom.items():
+        if len(values) == 1:
+            result.append(values[0])
+            continue
+        else:
+            value = sum([shift.value for shift in values]) / len(values)
+            result.append(ShiftData(values[0].atom, round(value, 3), UNUSED))
+
+    return result
+
+
 def _make_spectrum(
     shifts: List[ShiftData], info: PeakInfo, height: int = 1_000_000
 ) -> List[NewPeak]:
@@ -266,13 +207,14 @@ def _make_spectrum(
     # note this is not very sophisticated and doesn't cover side-chains... we ought to port the ccpn code
 
     updated_shifts = []
+
     for shift in shifts:
 
         sequence_code = str(shift.atom.residue.sequence_code)
         prefix = sequence_code[0] if sequence_code.startswith("@") else ""
         sequence_code = sequence_code.lstrip("@")
 
-        # TODO not sure this is the best way to do it, seperators are only relevant if the sequence code starts with @
+        # TODO not sure this is the best way to do it, separators are only relevant if the sequence code starts with @
         found_separator = False
         for separator in ["-", "+"]:
             if sequence_code.startswith(separator):
@@ -375,7 +317,9 @@ def _make_spectrum(
                     if shift.atom.atom_name == atom_name_required:
                         atom_set_shifts.append(shift)
 
-            if len(atom_set_shifts) != len(atom_set):
+            atom_set_shifts = _average_atom_set_shifts(atom_set_shifts)
+
+            if len(atom_set_shifts) > len(atom_set):
                 continue
 
             for i, shift in enumerate(atom_set_shifts):
@@ -407,3 +351,89 @@ def _make_spectrum(
                 peaks.append(new_peak)
 
     return peaks
+
+
+def _strs_to_experiment_types_or_exit_error(spectra):
+    result = []
+    spectrum_names = set([experiment.value for experiment in ExperimentType])
+    for spectum in spectra:
+
+        found = False
+        if spectum in spectrum_names:
+            result.append(ExperimentType[spectum])
+            found = True
+
+        experiment_types_upper = [
+            experiment_type.name.upper() for experiment_type in ExperimentType
+        ]
+        upper_experiments_to_experiments = {
+            experiment_type.name.upper(): experiment_type
+            for experiment_type in ExperimentType
+        }
+        counts = Counter(experiment_types_upper)
+
+        if not found:
+            spectrum_upper = spectum.upper()
+            if spectrum_upper in experiment_types_upper and counts[spectrum_upper] == 1:
+                result.append(upper_experiments_to_experiments[spectrum_upper])
+                found = True
+            if spectrum_upper in experiment_types_upper and counts[spectrum_upper] > 1:
+                degenerate_spectra = [
+                    experiment_type.name
+                    for experiment_type in ExperimentType
+                    if experiment_type.upper() == spectrum_upper
+                ]
+                _exit_degnerate_spectrum_type(
+                    spectum, list(ExperimentType), degenerate_spectra
+                )
+
+        if not found:
+            _exit_bad_spectrum_type(spectum, list(ExperimentType))
+
+    return result
+
+
+def _update_spectra_with_groups(spectra):
+
+    spectra = set(spectra)
+    to_update = []
+    for experiment_type in spectra:
+        if experiment_type in EXPERIMENT_GROUPS:
+            to_update.append(experiment_type)
+
+    for experiment_type in to_update:
+        spectra.update(EXPERIMENT_GROUPS[experiment_type])
+        spectra.remove(experiment_type)
+
+    return spectra
+
+
+def _exit_bad_spectrum_type(spectrum, known_spectra):
+    msg = """
+        the spectrum type {spectrum} is not know, available spectrum types are
+
+        {spectrum_types}
+    """
+    msg = dedent(msg)
+
+    spectrum_types = strings_to_tabulated_terminal_sensitive(known_spectra)
+    spectrum_types = indent(spectrum_types, FOUR_SPACES)
+
+    exit_error(f(msg))
+
+
+def _exit_degnerate_spectrum_type(spectrum, known_spectra, degenerate_spectra):
+    degenerate_spectra = ", ".join(degenerate_spectra)
+
+    spectrum_types = strings_to_tabulated_terminal_sensitive(known_spectra)
+    spectrum_types = indent(spectrum_types, FOUR_SPACES)
+
+    msg = f"""
+            the spectrum type {spectrum} doesn't exactly match a spectrum type in the list below
+            and has more than possible match when case is ignored [{degenerate_spectra}]
+
+            {spectrum_types}
+        """
+    msg = dedent(msg)
+
+    exit_error(f(msg))
