@@ -69,7 +69,7 @@ class DbFile:
 
 def _raise_data_before_format(line_info):
     msg = f"""\
-            bad nmrpipe db file, data seen before VAR and FORMAT
+            this is a bad nmrpipe db file, a data line was seen before VARS and FORMAT
             file: {line_info.file_name}'
             line no: {line_info.line_no}
             line: {line_info.line}
@@ -521,7 +521,11 @@ def _get_axis_labels(gdb_file):
 
 
 def get_gdb_columns(gdb_file: DbFile) -> List[str]:
-    return select_records(gdb_file, VARS)[0].values
+    result = ()
+    raw_columns = select_records(gdb_file, VARS)
+    if raw_columns:
+        result = raw_columns[0].values
+    return result
 
 
 def check_is_peak_file(gdb_file):
