@@ -8,6 +8,7 @@ from typing import List, Union
 import hjson
 import typer
 from fyeah import f
+from lazy_import import lazy_module
 from ordered_set import OrderedSet
 from pynmrstar import Entry
 from tabulate import tabulate
@@ -25,7 +26,6 @@ from nef_pipelines.lib.sequence_lib import (
 )
 from nef_pipelines.lib.shift_lib import shifts_to_nef_frame
 from nef_pipelines.lib.structures import AtomLabel, Residue, ShiftData, ShiftList
-from nef_pipelines.lib.translation.chem_comp import ID, ChemCompLinking
 from nef_pipelines.lib.util import (
     STDIN,
     chunks,
@@ -36,6 +36,8 @@ from nef_pipelines.lib.util import (
 )
 from nef_pipelines.transcoders.nmrstar import import_app
 
+chem_comp_module = lazy_module("nef_pipelines.lib.translation.chem_comp")
+
 
 class StereoAssignmentHandling(Enum):
     ALL_AMBIGUOUS = "all-ambiguous"
@@ -198,7 +200,7 @@ def _get_chem_atom_set_atoms(chem_atom_set, chem_comp):
         chem_atom_set.ID: chem_atom_set for chem_atom_set in chem_comp.chemAtomSets
     }
 
-    if isinstance(chem_atom_set, ID):
+    if isinstance(chem_atom_set, chem_comp_module.ID):
         result = [chem_atom_set]
     else:
         result = list(chem_atom_set.chemAtoms) if chem_atom_set.chemAtoms else []
@@ -402,10 +404,10 @@ def _get_atom_sets_by_residue(residue_names):
 
 
 LINKING_BY_NAME = {
-    "free": ChemCompLinking.free,
-    "start": ChemCompLinking.start,
-    "middle": ChemCompLinking.middle,
-    "end": ChemCompLinking.end,
+    "free": chem_comp_module.ChemCompLinking.free,
+    "start": chem_comp_module.ChemCompLinking.start,
+    "middle": chem_comp_module.ChemCompLinking.middle,
+    "end": chem_comp_module.ChemCompLinking.end,
 }
 NAME_BY_LINKING = {linking: name for name, linking in LINKING_BY_NAME.items()}
 
