@@ -12,12 +12,12 @@ from nef_pipelines.lib.nef_lib import (
     read_entry_from_file_or_stdin_or_exit_error,
 )
 from nef_pipelines.lib.sequence_lib import (
-    frame_to_chains,
+    chains_from_frames,
     get_chain_starts,
     get_residue_name_from_lookup,
     sequence_from_entry_or_exit,
-    sequence_from_frame,
     sequence_to_residue_name_lookup,
+    sequences_from_frames,
 )
 from nef_pipelines.lib.structures import AtomLabel, SequenceResidue, ShiftData
 from nef_pipelines.lib.util import (
@@ -79,7 +79,7 @@ def shifts(
 
     molecular_system_frame = molecular_system_from_entry_or_exit(entry)
 
-    chain_codes = frame_to_chains(molecular_system_frame)
+    chain_codes = chains_from_frames(molecular_system_frame)
 
     _exit_if_chain_selector_not_in_chain_codes(chain_codes, chain_selectors)
 
@@ -270,7 +270,7 @@ def _nmrview_sequences_from_entry(entry, chain_codes):
 
     molecular_system_frame = molecular_system_from_entry(entry)
 
-    residues = sequence_from_frame(molecular_system_frame)
+    residues = sequences_from_frames(molecular_system_frame)
 
     return _nef_to_nmrview_sequences(residues, chain_codes)
 
