@@ -1,8 +1,12 @@
 import os
 import re
+import sys
 import tempfile
+from dataclasses import dataclass
 from enum import Enum, auto
 from pathlib import Path
+from textwrap import dedent
+from typing import Optional
 from urllib.parse import urlsplit
 
 import requests
@@ -10,6 +14,7 @@ import typer
 from bs4 import BeautifulSoup
 from bs4.element import Comment
 from pynmrstar import Entry
+from tabulate import tabulate
 
 from nef_pipelines.lib.nef_lib import (
     UNUSED,
@@ -20,11 +25,65 @@ from nef_pipelines.lib.sequence_lib import sequences_from_frames, translate_1_to
 from nef_pipelines.lib.shift_lib import shifts_to_nef_frame
 from nef_pipelines.lib.structures import AtomLabel, Residue, ShiftData, ShiftList
 from nef_pipelines.lib.util import STDIN, exit_error, is_int
-from nef_pipelines.tools.chains.renumber import pipe as renumber
 from nef_pipelines.tools.loops.trim import ChainBound
 from nef_pipelines.tools.loops.trim import pipe as trim
 from nef_pipelines.transcoders.shiftx2 import import_app
 
+NETWORK_200_OK = 200
+
+PDB_UNIPROT_MAPPING_URL_TEMPLATE = (
+    "https://www.ebi.ac.uk/pdbe/api/mappings/uniprot/{code_or_filename}"
+)
+ALPHA_FOLD_KEY = "AIzaSyCeurAJz7ZGjPQUtEaerUkBZ3TaBkXrY94"
+ALPHA_FOLD_URL_TEMPLATE = (
+    "https://alphafold.ebi.ac.uk/api/prediction/{uniprot_id}?key={alphafold_key}"
+)
+
+SHIFTX2_RETRY_COUNT = 10
+
+
+@dataclass
+class NetworkResult:
+    network_ok: bool
+
+
+@dataclass
+class PdbAndUniprotIDs:
+    uniprot_id: str
+    pdb_code: str
+    chain_code: str
+
+
+@dataclass
+class PbdUniprotChainMapping:
+    pdb_uniprot_start: Optional[int]
+    pdb_uniprot_end: Optional[int]
+    pdb_start_residue: Optional[int]
+
+    @property
+    def pdb_uniprot_length(self):
+        return self.pdb_uniprot_end - self.pdb_uniprot_start
+
+
+@dataclass
+class PdbUniprotMapping(NetworkResult, PdbAndUniprotIDs, PbdUniprotChainMapping):
+    raw_data: Optional[dict]
+
+
+@dataclass
+class AlphafoldResult(NetworkResult, PdbAndUniprotIDs, PbdUniprotChainMapping):
+    pdb_url: Optional[str]
+    alphafold_id: Optional[str]
+    raw_data: Optional[dict]
+
+
+@dataclass
+class PDBDownloadResult(NetworkResult, PdbAndUniprotIDs, PbdUniprotChainMapping):
+    file_system_ok: bool
+    pdb_url: str
+    alphafold_id: str
+    pdb_file_path: Optional[Path]
+
 
 def tag_visible(element):
     if element.parent.name in [
@@ -70,6 +129,7 @@ def shifts(
     chain: str = typer.Option(
         None, "-c", "--chain", help="chain to label output shift data with"
     ),
+    verbose: bool = typer.Option(False, "--verbose", "-v", help="verbose output"),
     in_file: Path = typer.Option(
         STDIN,
         "-i",
@@ -79,12 +139,17 @@ def shifts(
 ):
     """- read a shiftx2 chemical shift prediction [alpha]"""
     entry = read_or_create_entry_exit_error_on_bad_file(in_file, "shiftx2")
-    entry = pipe(entry, code_or_file_name, source_chain, chain, alphafold)
+    entry = pipe(entry, code_or_file_name, source_chain, chain, alphafold, verbose)
     print(entry)
 
 
 def pipe(
-    entry: Entry, code_or_filename: str, source_chain: str, chain: str, alphafold: bool
+    entry: Entry,
+    code_or_filename: str,
+    source_chain: str,
+    chain: str,
+    alphafold: bool,
+    verbose: bool,
 ) -> Entry:
 
     file_path = Path(code_or_filename)
@@ -93,21 +158,33 @@ def pipe(
     else:
 
         if alphafold:
-            code_or_filename, uniprot_start, pdb_start, length = (
-                _pdb_code_to_uniprot_id(code_or_filename)
+            pdb_file_info = _pdb_code_to_alphafold_pdb_file(
+                code_or_filename, source_chain, verbose
             )
+            code_or_filename = pdb_file_info.pdb_file_path
             use_file = True
         else:
             use_file = False
 
         if not chain and source_chain:
             chain = source_chain
-        shifts = _get_shifts_from_server(
-            code_or_filename, source_chain, chain, use_file=use_file
+
+        for i in range(1, SHIFTX2_RETRY_COUNT + 1):
+            shifts = _get_shifts_from_server(
+                code_or_filename, source_chain, chain, use_file=use_file
+            )
+            if shifts:
+                break
+            elif verbose:
+                print(f"retrying shiftx2 ...[{i}]", file=sys.stderr)
+
+        _exit_if_too_many_attempted_connections(
+            shifts, code_or_filename, SHIFTX2_RETRY_COUNT
         )
 
     shift_list = ShiftList(shifts)
     frame = shifts_to_nef_frame(shift_list, "shiftx2")
+
     entry.add_saveframe(frame)
 
     if not chain and source_chain:
@@ -116,36 +193,54 @@ def pipe(
     if not chain and not source_chain:
         chain = "A"
 
-    sequence = sequences_from_frames(frame, chain)
-    sequence_start = min(
-        [residue.sequence_code for residue in sequence if is_int(residue.sequence_code)]
-    )
-    sequence_end = max(
-        [residue.sequence_code for residue in sequence if is_int(residue.sequence_code)]
-    )
-
     if alphafold:
-        chain_bounds = []
-        if uniprot_start > pdb_start:
+        sequence = sequences_from_frames(frame, chain)
+        shiftx2_uniprot_start = min(
+            [
+                residue.sequence_code
+                for residue in sequence
+                if is_int(residue.sequence_code)
+            ]
+        )
+        shiftx2_uniprot_end = max(
+            [
+                residue.sequence_code
+                for residue in sequence
+                if is_int(residue.sequence_code)
+            ]
+        )
 
-            chain_bounds.append(ChainBound(chain, sequence_start, uniprot_start - 1))
+        run_trim = False
+        if pdb_file_info.pdb_uniprot_start > shiftx2_uniprot_start:
+            start = pdb_file_info.pdb_uniprot_start
+            run_trim = True
+        else:
+            start = shiftx2_uniprot_start
 
-        if sequence_end > uniprot_start + length:
-            chain_bounds.append(
-                ChainBound(chain, uniprot_start + length + 1, sequence_end)
-            )
+        if pdb_file_info.pdb_uniprot_start < shiftx2_uniprot_end:
+            end = pdb_file_info.pdb_uniprot_end
+            run_trim = True
+        else:
+            end = shiftx2_uniprot_end
 
-        chain_bounds = {chain: chain_bounds}
+        if run_trim:
+            chain_bounds = [ChainBound(chain, start, end)]
 
-        entry = trim(entry, "shiftx2", SelectionType.NAME, chain_bounds)
+            chain_bounds = {chain: chain_bounds}
 
-        offset = pdb_start - uniprot_start + 1
-
-        entry = renumber(entry, "shiftx2", SelectionType.NAME, {chain: offset})
+            entry = trim(entry, "shiftx2", SelectionType.NAME, chain_bounds)
 
     return entry
 
 
+def _exit_if_too_many_attempted_connections(shifts, code_or_filename, RETRY_COUNT):
+    if not shifts:
+        msg = f"""
+            couldn't get a shiftx2 prediction for {code_or_filename} after {RETRY_COUNT} retries
+            """
+        exit_error(msg)
+
+
 class ShiftFormat(Enum):
     TABULAR = auto()
     CSV = auto()
@@ -173,6 +268,7 @@ def _read_shifts_from_file(file_path, cli_chain_code):
 
 
 def _parse_text_to_shifts(text, cli_chain_code, source):
+
     lines = []
     for i, line in enumerate(text.split("\n")):
         if i == 0:
@@ -358,12 +454,15 @@ def _get_shifts_from_server(
     _exit_if_error_calculating_shifts(pdb_file_or_code, r)
 
     soup = BeautifulSoup(r.text, features="html.parser")
-    link = soup.find_all("a", href=True, string="download predictions")[0]["href"]
-    data_url = f"{ROOT_URL}/{link}"
-    data_r = requests.get(data_url)
-    text = data_r.text
+    shifts = ""
+    putative_links = soup.find_all("a", href=True, string="download predictions")
+    if putative_links:
+        link = putative_links[0]["href"]
+        data_url = f"{ROOT_URL}/{link}"
+        data_r = requests.get(data_url)
+        text = data_r.text
 
-    shifts = _parse_text_to_shifts(text, cli_chain_code, CGI_URL)
+        shifts = _parse_text_to_shifts(text, cli_chain_code, CGI_URL)
 
     return shifts
 
@@ -441,88 +540,250 @@ def _get_filename_from_url(url=None):
     return os.path.basename(urlpath)
 
 
-def _pdb_code_to_uniprot_id(code_or_filename):
-    url = f"https://www.ebi.ac.uk/pdbe/api/mappings/uniprot/{code_or_filename}"
-    response = requests.get(url)
-
-    ok = True
-    if response.status_code != 200:
-        ok = False
-
-    if ok:
-        data = response.json()
-        if not data:
-            ok = False
-
-    if not ok:
+def _exit_if_alphafold_network_bad(alphafold_result):
+    if not alphafold_result.network_ok:
         msg = f"""
-        failed to get uniprot id from pdb code {code_or_filename}
-        is the network ok or could the pdb code be incorrect?
+           failed to get alphafold structure url using uniprot id {alphafold_result.uniprot_id} which was
+           mapped from pdb code {alphafold_result.pdb_code}. The url
+           {ALPHA_FOLD_URL_TEMPLATE.format(uniprot_id=alphafold_result.uniprot_id, alphafold_key=ALPHA_FOLD_KEY)}
+           could not be accessed is there a network problem?
         """
+
         exit_error(msg)
 
-    # more validation needed here
-    code_or_filename = code_or_filename.lower()
-    uniprot_id = next(iter(data[code_or_filename]["UniProt"].keys()))
-    chain_info = data[code_or_filename]["UniProt"][uniprot_id]
-    first_mapping = next(iter(chain_info["mappings"]))
 
-    uniprot_start = int(first_mapping["unp_start"])
-    uniprot_end = int(first_mapping["unp_end"])
-    pdb_start = int(
-        first_mapping["start"]["residue_number"]
-    )  # as opposed to 'author_residue_number'
-    length = uniprot_end - uniprot_start
+def _exit_if_pdb_download_bad(pdb_file_data: PDBDownloadResult):
 
-    key = "AIzaSyCeurAJz7ZGjPQUtEaerUkBZ3TaBkXrY94"
-    alphafold_url = f"https://alphafold.ebi.ac.uk/api/prediction/{uniprot_id}?key={key}"
+    msg = None
+    if not pdb_file_data.network_ok:
+        msg = f"""
+            The alphafold structure {pdb_file_data.alphafold_id} derived from {pdb_file_data.pdb_code} with chain
+            {pdb_file_data.chain_code} could not be download from the url:
+            {pdb_file_data.pdb_url}
+            is there a network problem
+            """
+
+    if not pdb_file_data.file_system_ok:
+        msg = f"""
+            The pdb file for the alphafold structure {pdb_file_data.alphafold_id} derived from {pdb_file_data.pdb_code}
+            with chain  {pdb_file_data.chain_code} could not be saved to the file system at the path, is your disk full?
+            """
+
+    if msg:
+        exit_error(msg)
+
+
+def _pdb_code_to_alphafold_pdb_file(code_or_filename, source_chain, verbose):
+    msg = [f"# alphafold: {code_or_filename}->"]
+
+    mapping = _convert_pdb_code_to_uniprot_id(code_or_filename, source_chain)
+
+    _exit_if_pdb_to_uniprot_network_failure(mapping)
+    _exit_if_pdb_to_uniprot_mapping_fails(mapping)
+
+    msg.append(f"{mapping.uniprot_id}->")
+
+    alphafold_result = _get_alphafold_pdb_url_from_uniprot_id(mapping)
+
+    _exit_if_alphafold_network_bad(alphafold_result)
+    _exit_if_alphafold_uniprot_to_pdb_url_fails(alphafold_result)
+
+    msg.append(alphafold_result.alphafold_id)
+
+    pdb_file_data = _download_pdb_file(alphafold_result)
+
+    _exit_if_pdb_download_bad(pdb_file_data)
+
+    # TODO: why does this corrupt a NEF stream i printed to stdout rather than stderr...
+    if verbose:
+        print(
+            "".join(msg),
+            f"[{ALPHA_FOLD_URL_TEMPLATE.format(uniprot_id=mapping.uniprot_id, alphafold_key=ALPHA_FOLD_KEY)}]",
+            file=sys.stderr,
+        )
+
+    return pdb_file_data
+
+
+def _download_pdb_file(alphafold_result: AlphafoldResult) -> PDBDownloadResult:
+
+    response = requests.get(alphafold_result.pdb_url)
+
+    network_ok = response.status_code == NETWORK_200_OK
+
+    data = response.text
+
+    if data:
+        pdb_filename = _get_filename_from_url(alphafold_result.pdb_url)
+        tmpdirname = tempfile.mkdtemp()
+        file_path = Path(tmpdirname) / pdb_filename
+        with open(file_path, "w") as fp:
+            fp.write(data)
+
+    result = PDBDownloadResult(
+        network_ok=network_ok,
+        file_system_ok=file_path.is_file(),
+        pdb_url=alphafold_result.pdb_url,
+        pdb_code=alphafold_result.pdb_code,
+        chain_code=alphafold_result.chain_code,
+        uniprot_id=alphafold_result.uniprot_id,
+        alphafold_id=alphafold_result.alphafold_id,
+        pdb_file_path=file_path,
+        pdb_uniprot_start=alphafold_result.pdb_uniprot_start,
+        pdb_uniprot_end=alphafold_result.pdb_uniprot_end,
+        pdb_start_residue=alphafold_result.pdb_start_residue,
+    )
+    return result
+
+
+def _get_alphafold_pdb_url_from_uniprot_id(
+    mapping: PdbUniprotMapping,
+) -> AlphafoldResult:
+
+    alphafold_url = ALPHA_FOLD_URL_TEMPLATE.format(
+        uniprot_id=mapping.uniprot_id, alphafold_key=ALPHA_FOLD_KEY
+    )
 
     response = requests.get(alphafold_url)
 
-    ok = True
-    if response.status_code != 200:
-        ok = False
+    network_ok = response.status_code == NETWORK_200_OK
 
-    if ok:
-        data = response.json()
-        if not data:
-            ok = False
+    raw_data = response.json() if network_ok and response.text else None
 
-    if not ok:
+    uniprot_id = mapping.uniprot_id
+    entry = None
+    alphafold_pdb_url = None
+    alphafold_entry_id = None
+    if raw_data:
+        entry = raw_data[0]
+        alphafold_pdb_url = entry["pdbUrl"]
+        alphafold_entry_id = entry["entryId"]
+
+    alphafold_result = AlphafoldResult(
+        network_ok=network_ok,
+        pdb_url=alphafold_pdb_url,
+        uniprot_id=uniprot_id,
+        pdb_code=mapping.pdb_code,
+        chain_code=mapping.chain_code,
+        raw_data=entry,
+        alphafold_id=alphafold_entry_id,
+        pdb_uniprot_start=mapping.pdb_uniprot_start,
+        pdb_uniprot_end=mapping.pdb_uniprot_end,
+        pdb_start_residue=mapping.pdb_start_residue,
+    )
+
+    return alphafold_result
+
+
+def _exit_if_alphafold_uniprot_to_pdb_url_fails(alphafold_result):
+    if not alphafold_result.pdb_url:
         msg = f"""
-           failed to get uniprot structure using pdb code {code_or_filename}
-           is the network ok or could the pdb code be incorrect or for a structure not in the embl database?
+           failed to get alphafold structure url using uniprot id {alphafold_result.uniprot_id} which was
+           mapped from pdb code {alphafold_result.pdb_code}. Is thisfor a structure not in the embl database
+           [eg a virus, or a bacterium like E.coli]?
            """
         exit_error(msg)
 
-    entry = data[0]
-    alphafold_pdb_url = entry["pdbUrl"]
 
-    # print(uniprot_id, data)
+def _convert_pdb_code_to_uniprot_id(
+    code_or_filename, source_chain
+) -> PdbUniprotMapping:
 
-    response = requests.get(alphafold_pdb_url)
+    response = requests.get(
+        PDB_UNIPROT_MAPPING_URL_TEMPLATE.format(code_or_filename=code_or_filename)
+    )
 
-    ok = True
-    if response.status_code != 200:
-        ok = False
+    network_ok = response.status_code == NETWORK_200_OK
 
-    if ok:
-        data = response.text
-        if not data:
-            ok = False
+    raw_mapping_data = response.json() if network_ok and response.text else None
 
-    if not ok:
+    uniprot_id = None
+    if raw_mapping_data:
+        code_or_filename = code_or_filename.lower()
+
+        for putative_uniprot_id, mapping_element in raw_mapping_data[code_or_filename][
+            "UniProt"
+        ].items():
+            for mapping in mapping_element["mappings"]:
+                if mapping["chain_id"] == source_chain:
+                    uniprot_id = putative_uniprot_id
+                    break
+
+    pdb_uniprot_start = None
+    pdb_uniprot_end = None
+    pdb_start_residue = None
+    if uniprot_id:
+
+        chain_info = raw_mapping_data[code_or_filename]["UniProt"][uniprot_id]
+        first_mapping = next(iter(chain_info["mappings"]))
+
+        pdb_uniprot_start = int(first_mapping["unp_start"])
+        pdb_uniprot_end = int(first_mapping["unp_end"])
+        pdb_start_residue = int(
+            first_mapping["start"]["residue_number"]
+        )  # as opposed to 'author_residue_number'
+
+    result = PdbUniprotMapping(
+        network_ok=network_ok,
+        pdb_code=code_or_filename,
+        chain_code=code_or_filename,
+        uniprot_id=uniprot_id,
+        pdb_uniprot_start=pdb_uniprot_start,
+        pdb_uniprot_end=pdb_uniprot_end,
+        pdb_start_residue=pdb_start_residue,
+        raw_data=raw_mapping_data,
+    )
+
+    return result
+
+
+def _exit_if_pdb_to_uniprot_network_failure(mapping: PdbUniprotMapping):
+
+    if not mapping.network_ok:
         msg = f"""
-               failed to get alphafold structure using pdb code {code_or_filename} mapped to uniprot id {uniprot_id}
-               is the network ok or could the pdb code be incorrect or for a structure not in the embl database?
-               """
+        failed to get uniprot id from pdb code {mapping.pdb_code}
+        it appears there was a network problem accessing the url:
+
+        {PDB_UNIPROT_MAPPING_URL_TEMPLATE.format(code_or_filename=mapping.pdb_code)}
+        """
         exit_error(msg)
 
-    pdb_filename = _get_filename_from_url(alphafold_pdb_url)
 
-    tmpdirname = tempfile.mkdtemp()
-    file_path = Path(tmpdirname) / pdb_filename
-    with open(file_path, "w") as fp:
-        fp.write(data)
+def _exit_if_pdb_to_uniprot_mapping_fails(mapping: PdbUniprotMapping):
 
-    return file_path, uniprot_start, pdb_start, length
+    if mapping.uniprot_id is None:
+        mapping_table = []
+        for putative_uniprot_id, mapping_element in mapping.raw_data[mapping.pdb_code][
+            "UniProt"
+        ].items():
+            for mapping in mapping_element["mappings"]:
+                mapping_table.append(
+                    (
+                        putative_uniprot_id,
+                        mapping["chain_id"],
+                        mapping["unp_start"],
+                        mapping["unp_end"],
+                    )
+                )
+
+        headings = [
+            "uniprot id",
+            "chain code",
+            "sequence code start",
+            "sequence code end",
+        ]
+        mapping_table = tabulate(mapping_table, headers=headings)
+
+        msg = """
+            using pdb code {code_or_filename} and chain {source_chain} I couldn't find a mapping,
+            the available mappings are
+
+            {mapping_table}
+            """
+        msg = dedent(msg)
+        msg = msg.format(
+            code_or_filename=mapping.pdb_code.upper(),
+            source_chain=mapping.chain_code,
+            mapping_table=mapping_table,
+        )
+        exit_error(msg)
