@@ -1,8 +1,9 @@
 import sys
 from pathlib import Path
-from typing import List
+from typing import Dict, List, Tuple
 
 import typer
+from fyeah import f
 from pynmrstar import Entry, Saveframe
 from tabulate import tabulate
 
@@ -10,20 +11,22 @@ from nef_pipelines.lib.nef_lib import (
     read_entry_from_stdin_or_exit,
     select_frames_by_name,
 )
-from nef_pipelines.lib.sequence_lib import TRANSLATIONS_3_1_PROTEIN
+from nef_pipelines.lib.sequence_lib import (
+    TRANSLATIONS_3_1_PROTEIN,
+    get_residue_name_from_lookup,
+    sequence_from_entry_or_exit,
+    sequence_to_residue_name_lookup,
+)
 from nef_pipelines.lib.shift_lib import nef_frames_to_shifts
 from nef_pipelines.lib.structures import ShiftData
 from nef_pipelines.lib.util import STDOUT, exit_error, flatten, is_int
 from nef_pipelines.transcoders.shifty import export_app
 
-OUTPUT_FILE_SHORTCUTS = """\
-    for stdout use -, ! will use the name of the nef entry in the
-    form <NEF-ENTRY-NAME>_shifts.shifty
-"""
-
-OUTPUT_FILE_HELP = f"""\
-    file name to output to [{OUTPUT_FILE_SHORTCUTS}]
-"""
+FILE_NAME_TEMPLATE = "{nef_entry_id}_{chain_code}.shifty"
+OUTPUT_FILE_HELP = (
+    f"file name to output to [default is {FILE_NAME_TEMPLATE}; - gives stdout]"
+)
+FILE_NAME_TEMPLATE_HELP = f"file name template default is [{FILE_NAME_TEMPLATE}]"
 
 SHIFTY_HEADINGS_LOOKUP = (
     ("HA", ("HA1", "HA")),
@@ -57,6 +60,12 @@ def shifts(
         help="chain to export [defaults to the first chain if only one present]",
         metavar="<CHAIN-CODE>",
     ),
+    file_name_template: str = typer.Option(
+        FILE_NAME_TEMPLATE, help=FILE_NAME_TEMPLATE_HELP
+    ),
+    infill: bool = typer.Option(
+        True, help="infill missing residues with values set to 0.0"
+    ),
     output_file: str = typer.Argument(
         None,
         help=OUTPUT_FILE_HELP,
@@ -69,16 +78,29 @@ def shifts(
         shift_frames = ["*"]
 
     entry = read_entry_from_stdin_or_exit()
+    nef_entry_id = entry.entry_id  # noqa: F841
+
+    sequence = sequence_from_entry_or_exit(entry) if infill else None
+    sequence_lookup = sequence_to_residue_name_lookup(sequence) if sequence else None
+
+    if chain_code is None:
+        chain_codes = set([residue.chain_code for residue in sequence])
+        assigned_chain_codes = set(
+            [chain_code for chain_code in chain_codes if chain_code not in ["@", "#"]]
+        )
+
+        if len(assigned_chain_codes) == 1:
+            chain_code = assigned_chain_codes.pop()
+        else:
+            msg = f"""you must chhose a single assigned chain i got {', '.join(assigned_chain_codes)}"""
+            exit_error(msg)
 
     if output_file is None:
-        msg = f"you need to specify an output file name, [{OUTPUT_FILE_SHORTCUTS}]"
+        output_file = f(file_name_template)
 
-        exit_error(msg)
-    output_file = (
-        f"{entry.entry_id}_shifts.shifty" if output_file == "!" else Path(output_file)
-    )
+    output_file = Path(output_file) if output_file != "-" else STDOUT
 
-    entry = pipe(entry, shift_frames, chain_code, output_file)
+    entry = pipe(entry, shift_frames, chain_code, infill, sequence_lookup, output_file)
 
     if not (sys.stdout.isatty() or output_file == STDOUT):
         print(entry)
@@ -88,6 +110,8 @@ def pipe(
     entry: Entry,
     shift_frames: List[Saveframe],
     selected_chain_code: str,
+    infill: bool,
+    residue_name_lookup: Dict[Tuple[str, str], str],
     output_file: Path,
 ) -> Entry:
 
@@ -133,6 +157,9 @@ def pipe(
         key = sequence_code, residue_name_1let
         lines.setdefault(key, {})[atom_name] = value
 
+    if infill:
+        lines = _infill_sequence_exit(lines, chain_code, residue_name_lookup)
+
     table = [[*SHIFTY_EXTRA_HEADINGS, *SHIFTY_ATOMS]]
 
     for (residue_number, residue_type), shifts in lines.items():
@@ -168,6 +195,42 @@ def pipe(
     return entry
 
 
+def _infill_sequence_exit(lines, chain_code, residue_name_lookup):
+    sequence_codes = [sequence_code for sequence_code, _ in lines.keys()]
+    max_sequence_code, min_sequence_code = _sequence_code_range_or_exit_on_non_ints(
+        chain_code, sequence_codes
+    )
+
+    for sequence_code in range(min_sequence_code, max_sequence_code + 1):
+        residue_name = get_residue_name_from_lookup(
+            chain_code, sequence_code, residue_name_lookup
+        )
+        residue_name_1_let = TRANSLATIONS_3_1_PROTEIN[residue_name]
+        key = sequence_code, residue_name_1_let
+        if key not in lines:
+            lines[key] = {atom_name: 0.0 for atom_name in SHIFTY_LOOKUP_ATOMS}
+    lines = dict(sorted(lines.items()))
+    return lines
+
+
+def _sequence_code_range_or_exit_on_non_ints(chain_code, sequence_codes):
+    int_sequence_codes = [
+        sequence_code for sequence_code in sequence_codes if is_int(sequence_code)
+    ]
+    min_sequence_code = min(int_sequence_codes)
+    max_sequence_code = max(int_sequence_codes)
+    is_int_list = [is_int(sequence_code) for sequence_code in sequence_codes]
+    all_ints = all(is_int_list)
+    if not all_ints:
+        msg = f"""\
+            the sequence codes for chain {chain_code} aren't all integers
+            the sequence codes must be integers to infill missing residues
+            the first non-integer sequence code is {min_sequence_code + sequence_codes[is_int_list.index(False)]}
+        """
+        exit_error(msg)
+    return max_sequence_code, min_sequence_code
+
+
 def _is_pseudo_residue(chain_code, sequence_code):
     return (
         chain_code.startswith("@-")
@@ -212,7 +275,7 @@ def _get_chain_code_or_exit(
     if selected_chain_code not in chain_codes:
         msg = f"""\
             the chain code you selected [{selected_chain_code}] isn't in the chain codes [{' '.join(chain_codes)}]
-            in the entry {entry.name}
+            in the entry {entry.entry_id}
         """
         exit_error(msg)
 
